@@ -1,10 +1,9 @@
 use crate::*;
-use boco::*;
 use err_tools::*;
 use std::ops::Deref;
 use tparam::*;
 
-pub fn as_base64<'a>(l: &[TBoco<'a>]) -> anyhow::Result<TBoco<'a>> {
+pub fn as_base64<'a>(l: &[TCow<'a>]) -> anyhow::Result<TCow<'a>> {
     let b_ar = match l.get(0).e_str("needs 1 arg")?.deref() {
         TData::String(s) => s.as_bytes(),
         TData::Bytes(b) => b,
@@ -14,7 +13,7 @@ pub fn as_base64<'a>(l: &[TBoco<'a>]) -> anyhow::Result<TBoco<'a>> {
     b_ok(TData::String(s))
 }
 
-pub fn from_base64<'a>(l: &[TBoco<'a>]) -> anyhow::Result<TBoco<'a>> {
+pub fn from_base64<'a>(l: &[TCow<'a>]) -> anyhow::Result<TCow<'a>> {
     let b_ar = match l.get(0).e_str("needs 1 arg")?.deref() {
         TData::String(s) => s.as_bytes(),
         TData::Bytes(b) => b,
