@@ -41,18 +41,24 @@ pub enum TreeItem {
         p: Expr,
         b: Block,
     },
-    Define(String, Block),
-    Global(String, Block),
+    Define(String, Vec<String>, Block),
+    Global(String, Vec<String>, Block),
     Let(Vec<(String, Expr)>),
     Export(Vec<(String, Expr)>),
     AtLet(String, Block),
     AtExport(String, Block),
     Return(Expr),
     Switch(Vec<Expr>, Vec<Case>),
+    As {
+        exp: Expr,
+        pat: pattern::Pattern,
+        block: Block,
+    },
 }
 
 #[derive(Clone, Debug, PartialEq)]
 pub struct TreeTemplate {
+    pub pnames: Vec<String>,
     pub v: Vec<TreeItem>,
 }
 
@@ -67,8 +73,8 @@ pub enum FlatItem {
     For(String, String, Expr),
     AtLet(String),
     AtExport(String),
-    Define(String),
-    Global(String),
+    Define(String, Vec<String>),
+    Global(String, Vec<String>),
     Let(Vec<(String, Expr)>),
     Export(Vec<(String, Expr)>),
     Block(String, Vec<Expr>),
@@ -76,6 +82,7 @@ pub enum FlatItem {
     Return(Expr),
     Switch(Vec<Expr>),
     Case(Vec<pattern::Pattern>),
+    As(Expr, pattern::Pattern),
 }
 
 #[derive(Clone, Debug, PartialEq)]
@@ -215,12 +222,24 @@ impl TreeItem {
                 scope.set_root(name, TData::String(ch));
                 Ok(String::new())
             }
-            TreeItem::Define(name, block) => {
-                scope.set(name, TData::Template(TreeTemplate { v: block.clone() }));
+            TreeItem::Define(name, pnames, block) => {
+                scope.set(
+                    name,
+                    TData::Template(TreeTemplate {
+                        v: block.clone(),
+                        pnames: pnames.clone(),
+                    }),
+                );
                 Ok(String::new())
             }
-            TreeItem::Global(name, block) => {
-                tm.insert_t(name.to_string(), TreeTemplate { v: block.clone() });
+            TreeItem::Global(name, pnames, block) => {
+                tm.insert_t(
+                    name.to_string(),
+                    TreeTemplate {
+                        v: block.clone(),
+                        pnames: pnames.clone(),
+                    },
+                );
                 Ok(String::new())
             }
             TreeItem::Switch(params, cases) => {
@@ -253,6 +272,16 @@ impl TreeItem {
                 }
                 return Ok(String::new());
             }
+            TreeItem::As { exp, pat, block } => {
+                scope.push();
+                let val = exp.run(scope, tm, fm)?.into_owned();
+                if pat.match_data(&val, scope, tm, fm) {
+                    let r = run_block(block, scope, tm, fm)?;
+                    scope.pop();
+                    return Ok(r);
+                }
+                return Ok(String::new());
+            }
         }
     }
 }
@@ -329,8 +358,8 @@ pub fn tt_basic<I: Iterator<Item = FlatItem>>(
         FlatItem::Export(v) => TreeItem::Export(v),
         FlatItem::AtExport(v) => TreeItem::AtExport(v, tt_name_block("export", it)?),
 
-        FlatItem::Define(v) => TreeItem::Define(v, tt_name_block("define", it)?),
-        FlatItem::Global(v) => TreeItem::Global(v, tt_name_block("global", it)?),
+        FlatItem::Define(v, params) => TreeItem::Define(v, params, tt_name_block("define", it)?),
+        FlatItem::Global(v, params) => TreeItem::Global(v, params, tt_name_block("global", it)?),
         FlatItem::If(p) => tt_if_yes(p, it)?,
         FlatItem::For(k, v, p) => TreeItem::For {
             k,
@@ -346,6 +375,10 @@ pub fn tt_basic<I: Iterator<Item = FlatItem>>(
                 block,
             }
         }
+        FlatItem::As(exp, pat) => {
+            let block = tt_name_block("as", it)?;
+            TreeItem::As { exp, pat, block }
+        }
         FlatItem::Return(p) => TreeItem::Return(p),
         FlatItem::Comment => TreeItem::Comment,
         FlatItem::Switch(p) => return tt_switch(p, it),
@@ -390,7 +423,10 @@ pub fn tt_root_block<I: Iterator<Item = FlatItem>>(i: &mut I) -> anyhow::Result<
     while let Some(t) = i.next() {
         res.push(tt_basic(t, i)?)
     }
-    Ok(TreeTemplate { v: res })
+    Ok(TreeTemplate {
+        v: res,
+        pnames: Vec::new(),
+    })
 }
 
 pub fn tt_name_block<I: Iterator<Item = FlatItem>>(name: &str, i: &mut I) -> anyhow::Result<Block> {
